@@ -2,10 +2,13 @@
 #  Cython - Compilation-wide options and pragma declarations
 #
 
-cache_builtins = 1  #  Perform lookups on builtin names only once
+# Perform lookups on builtin names only once, at module initialisation
+# time.  This will prevent the module from getting imported if a
+# builtin name that it uses cannot be found during initialisation.
+cache_builtins = True
 
-embed_pos_in_docstring = 0
-gcc_branch_hints = 1
+embed_pos_in_docstring = False
+gcc_branch_hints = True
 
 pre_import = None
 docstrings = True
@@ -14,14 +17,23 @@ docstrings = True
 # 0: None, 1+: interned objects, 2+: cdef globals, 3+: types objects
 # Mostly for reducing noise for Valgrind, only executes at process exit
 # (when all memory will be reclaimed anyways).
-generate_cleanup_code = 0
+generate_cleanup_code = False
 
-annotate = 0
+annotate = False
 
 # This will abort the compilation on the first error occured rather than trying
 # to keep going and printing further error messages.
 fast_fail = False
 
+# Make all warnings into errors.
+warning_errors = False
+
+# Make unknown names an error.  Python raises a NameError when
+# encountering unknown names at runtime, whereas this option makes
+# them a compile time error.  If you want full Python compatibility,
+# you should disable this option and also 'cache_builtins'.
+error_on_unknown_names = True
+
 # This will convert statements of the form "for i in range(...)"
 # to "for i from ..." when i is a cdef'd integer type, and the direction
 # (i.e. sign of step) can be determined.
@@ -29,37 +41,41 @@ fast_fail = False
 # i to overflow. Specifically, if this option is set, an error will be
 # raised before the loop is entered, wheras without this option the loop
 # will execute until an overflowing value is encountered.
-convert_range = 1
+convert_range = True
 
 # Enable this to allow one to write your_module.foo = ... to overwrite the
 # definition if the cpdef function foo, at the cost of an extra dictionary
 # lookup on every call.
 # If this is 0 it simply creates a wrapper.
-lookup_module_cpdef = 0
-
-# This will set local variables to None rather than NULL which may cause
-# surpress what would be an UnboundLocalError in pure Python but eliminates
-# checking for NULL on every use, and can decref rather than xdecref at the end.
-# WARNING: This is a work in progress, may currently segfault.
-init_local_none = 1
-
-# Append the c file and line number to the traceback for exceptions.
-c_line_in_traceback = 1
+lookup_module_cpdef = False
 
 # Whether or not to embed the Python interpreter, for use in making a
-# standalone executable. This will provide a main() method which simply
+# standalone executable or calling from external libraries.
+# This will provide a method which initalizes the interpreter and
 # executes the body of this module.
-embed = False
+embed = None
 
 # Disables function redefinition, allowing all functions to be declared at
-# module creation time. For legacy code only. 
+# module creation time. For legacy code only, needed for some circular imports.
 disable_function_redefinition = False
 
+# In previous iterations of Cython, globals() gave the first non-Cython module
+# globals in the call stack.  Sage relies on this behavior for variable injection.
+old_style_globals = False
+
+# Allows cimporting from a pyx file without a pxd file.
+cimport_from_pyx = False
+
+
+# max # of dims for buffers -- set lower than number of dimensions in numpy, as
+# slices are passed by value and involve a lot of copying
+buffer_max_dims = 8
 
 # Declare compiler directives
 directive_defaults = {
     'boundscheck' : True,
     'nonecheck' : False,
+    'initializedcheck' : True,
     'embedsignature' : False,
     'locals' : {},
     'auto_cpdef': False,
@@ -80,16 +96,43 @@ directive_defaults = {
     'autotestdict.all': False,
     'language_level': 2,
     'fast_getattr': False, # Undocumented until we come up with a better way to handle this everywhere.
+    'py2_import': False, # For backward compatibility of Cython's source code in Py3 source mode
+
+    # set __file__ and/or __path__ to known source/target path at import time (instead of not having them available)
+    'set_initial_path' : None,  # SOURCEFILE or "/full/path/to/module"
 
     'warn': None,
     'warn.undeclared': False,
+    'warn.unreachable': True,
+    'warn.maybe_uninitialized': False,
+    'warn.unused': False,
+    'warn.unused_arg': False,
+    'warn.unused_result': False,
+
+# optimizations
+    'optimize.inline_defnode_calls': False,
+
+# remove unreachable code
+    'remove_unreachable': True,
+
+# control flow debug directives
+    'control_flow.dot_output': "", # Graphviz output filename
+    'control_flow.dot_annotate_defs': False, # Annotate definitions
 
 # test support
     'test_assert_path_exists' : [],
     'test_fail_if_path_exists' : [],
 
 # experimental, subject to change
-    'binding': False,
+    'binding': None,
+    'experimental_cpp_class_def': False
+}
+
+# Extra warning directives
+extra_warnings = {
+    'warn.maybe_uninitialized': True,
+    'warn.unreachable': True,
+    'warn.unused': True,
 }
 
 # Override types possibilities above, if needed
@@ -97,6 +140,12 @@ directive_types = {
     'final' : bool,  # final cdef classes and methods
     'internal' : bool,  # cdef class visibility in the module dict
     'infer_types' : bool, # values can be True/None/False
+    'binding' : bool,
+    'cfunc' : None, # decorators do not take directive value
+    'ccall' : None,
+    'cclass' : None,
+    'returns' : type,
+    'set_initial_path': str,
     }
 
 for key, val in directive_defaults.items():
@@ -105,11 +154,12 @@ for key, val in directive_defaults.items():
 
 directive_scopes = { # defaults to available everywhere
     # 'module', 'function', 'class', 'with statement'
-    'final' : ('cclass',),   # add 'method' in the future
+    'final' : ('cclass', 'function'),
     'internal' : ('cclass',),
     'autotestdict' : ('module',),
     'autotestdict.all' : ('module',),
     'autotestdict.cdef' : ('module',),
+    'set_initial_path' : ('module',),
     'test_assert_path_exists' : ('function', 'class', 'cclass'),
     'test_fail_if_path_exists' : ('function', 'class', 'cclass'),
 }
